function implode(glue, pieces) {
    //  discuss at: http://phpjs.org/functions/implode/
    // original by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
    // improved by: Waldo Malqui Silva
    // improved by: Itsacon (http://www.itsacon.net/)
    // bugfixed by: Brett Zamir (http://brett-zamir.me)
    //   example 1: implode(' ', ['Kevin', 'van', 'Zonneveld']);
    //   returns 1: 'Kevin van Zonneveld'
    //   example 2: implode(' ', {first:'Kevin', last: 'van Zonneveld'});
    //   returns 2: 'Kevin van Zonneveld'
    var i = '',
        retVal = '',
        tGlue = '';

    if (typeof pieces === 'object') {
        if (Object.prototype.toString.call(pieces) === '[object Array]') {
            return pieces.join(glue);
        }
        for (i in pieces) {
            retVal += tGlue + pieces[i];
            tGlue = glue;
        }
        return retVal;
    }
    return pieces;
}

function splitString(string, split_length) {
    //  discuss at: http://phpjs.org/functions/splitString/
    // original by: Martijn Wieringa
    // improved by: Brett Zamir (http://brett-zamir.me)
    // bugfixed by: Onno Marsman
    //  revised by: Theriault
    //  revised by: Rafal Kukawski (http://blog.kukawski.pl/)
    //    input by: Bjorn Roesbeke (http://www.bjornroesbeke.be/)
    //   example 1: splitString('Hello Friend', 3);
    //   returns 1: ['Hel', 'lo ', 'Fri', 'end']
    if (split_length === null) {
        split_length = 1;
    }
    if (string === null || split_length < 1) {
        return false;
    }
    string += '';
    var chunks = [],
        pos = 0,
        len = string.length;
    while (pos < len) {
        chunks.push(string.slice(pos, pos += split_length));
    }
    return chunks;
}

function encrypt(msg, key) {
    //rc4
    //http://pongwar.com/arc4/
    var result = '';
    try {
        result = strToHex(cipher(msg, key));
    } catch (ex) {
        result = 'ERROR: ' + ex;
    }
    return result;
}

function cipher(msg, key) {
    var state = initState(key);
    var x = 0;
    var y = 0;
    var temp = 0;
    var output = '';
    for (var i = 0; i < msg.length; i = i + 1) {
        x = (x + 1) % 256;
        y = (state[x] + y) % 256;
        temp = state[x];
        state[x] = state[y];
        state[y] = temp;
        output = output + String.fromCharCode(msg.charCodeAt(i) ^ state[(state[x] + state[y]) % 256]);
    }
    return output;
}

function initState(key) {
    var state = new Array(255);
    if (key.length > 0) {
        var j = 0;
        var temp = 0;
        for (var i = 0; i <= 255; i = i + 1) {
            state[i] = i;
        }
        for (var i = 0; i <= 255; i = i + 1) {
            j = (j + state[i] + key.charAt(i % key.length).charCodeAt(0)) % 256;
            temp = state[i];
            state[i] = state[j];
            state[j] = temp;
        }
    } else {
        throw ('Blank Key')
    }
    return state;
}

function strToHex(str) {
    var hex = '';
    for (var i = 0; i < str.length; i = i + 1) {
        hex = hex + decToHex(str.charCodeAt(i));
    }
    return hex;
}

function decToHex(dec) {
    var hex = dec.toString(16);
    if (hex.length < 2) {
        hex = '0' + hex;
    } else if (hex.length > 2) {
        throw ('Unsupported Input')
    }
    return hex;
}

function invertArray(array) {
    // converts string or number to an array and inverts it
    var res = [];
    var cad = array.toString();
    for(var i=cad.length-1; i>=0; i--){
        res = res  + [cad.charAt(i)];
    }

    return res;
};

var Verhoeff = {
    //Verhoweff algorithm: http://en.wikipedia.org/wiki/Verhoeff_algorithm

    // multiplication table d
    'd': [
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
        [1, 2, 3, 4, 0, 6, 7, 8, 9, 5],
        [2, 3, 4, 0, 1, 7, 8, 9, 5, 6],
        [3, 4, 0, 1, 2, 8, 9, 5, 6, 7],
        [4, 0, 1, 2, 3, 9, 5, 6, 7, 8],
        [5, 9, 8, 7, 6, 0, 4, 3, 2, 1],
        [6, 5, 9, 8, 7, 1, 0, 4, 3, 2],
        [7, 6, 5, 9, 8, 2, 1, 0, 4, 3],
        [8, 7, 6, 5, 9, 3, 2, 1, 0, 4],
        [9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
    ],

    // permutation table p
    'p': [
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
        [1, 5, 7, 6, 2, 8, 3, 0, 9, 4],
        [5, 8, 0, 3, 7, 9, 6, 1, 4, 2],
        [8, 9, 1, 6, 0, 4, 3, 5, 2, 7],
        [9, 4, 5, 3, 1, 2, 6, 8, 7, 0],
        [4, 2, 8, 6, 5, 7, 3, 9, 0, 1],
        [2, 7, 9, 3, 8, 0, 6, 4, 1, 5],
        [7, 0, 4, 6, 9, 1, 3, 2, 5, 8]
    ],

    // inverse table inv
    'inv': [0, 4, 3, 2, 1, 5, 6, 7, 8, 9],

    // generates and returns one checksum digit
    'digit': function(array) {

        var c = 0;
        var invertedArray = invertArray(array);

        for (var i = 0; i < invertedArray.length; i++) {
            c = Verhoeff.d[c][Verhoeff.p[((i + 1) % 8)][invertedArray[i]]];
        }

        return Verhoeff.inv[c];
    },

    // validates checksum
    'validate': function(array) {

        var c = 0;
        var invertedArray = invertArray(array);

        for (var i = 0; i < invertedArray.length; i++) {
            c = Verhoeff.d[c][Verhoeff.p[(i % 8)][invertedArray[i]]];
        }

        return (c === 0);
    },

    // generates nroDigits Verhoeff digits and returns the same array with the two digits apended at the end
    'addDigits': function(array, nroDigits) {
        var r = array.toString();

        for (var i = 0; i < parseInt(nroDigits); i++) {
            r = r + Verhoeff.digit(r).toString();
        }
        return r;
    }
}

function getBase64(n) {
    var d = [
        '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
        'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
        'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '+', '/'
    ];

    var c = 1;
    var r = '';
    while (c > 0) {
        c = Math.floor(n / 64);
        r = d[n % 64] + r;
        n = c;
    }
    return r;
}

function generate(numautorizacion, numfactura, nitcliente, fecha, monto, clave) {
	//numautorizacion: string
	//numfactura: string
	//nitcliente: string
	//fecha:string date with format YYYYMMDD
	//monto: numeric
	//clave: string

    var decimal = monto - Math.floor(monto);
    if (decimal >= 0.5) {
        monto = Math.floor(monto) + 1;
    } else {
        monto = Math.floor(monto);
    }

    numfactura = Verhoeff.addDigits(numfactura, 2);
    nitcliente = Verhoeff.addDigits(nitcliente, 2);
    fecha = Verhoeff.addDigits(fecha, 2);
    monto = Verhoeff.addDigits(monto, 2);

    var suma = parseInt(numfactura) + parseInt(nitcliente) + parseInt(fecha) + parseInt(monto);
    suma = Verhoeff.addDigits(suma.toString(), 5);

    var lastFiveVerhoeffDigits = suma.substring(suma.length - 5);

/*
    console.log("clave:", clave);
    console.log("numfactura:", numfactura);
    console.log("nit:", nitcliente);
    console.log("fecha:", fecha);
    console.log("monto:", monto);
    console.log("suma:", suma);
    console.log("Last 5:", lastFiveVerhoeffDigits);
*/

    var cads = [numautorizacion, numfactura, nitcliente, fecha, monto];
    var msg = '';
    var p = 0;
    for (var i = 0; i < 5; i++) {
        var x = 1 + parseInt(lastFiveVerhoeffDigits.charAt(i));
        msg = msg + cads[i] + clave.substr(p, x);
        p = p + x;
    }
    var codif = encrypt(msg, clave + lastFiveVerhoeffDigits);
    codif = codif.toUpperCase();
    var st = 0;
    var sp = [0, 0, 0, 0, 0];
    var codif_length = codif.length;
    for (var i = 0; i < codif_length; i++) {
        st = st + codif.charCodeAt(i);
        sp[i % 5] = sp[i % 5] + codif.charCodeAt(i);
    }
    var stt = 0;
    for (var i = 0; i < 5; i++) {
        stt += Math.floor((st * sp[i]) / (1 + parseInt(lastFiveVerhoeffDigits.charAt(i))));
    }
    //console.log("Sumatoria producto: " + stt); 

    var base64 = getBase64(stt.toString());
    //console.log("Base64: ", base64); 

    var result = implode('-', splitString(encrypt(base64, clave + lastFiveVerhoeffDigits).toUpperCase(), 2));

    return result;
}

function testCodigoControl(numautorizacion, numfactura, nitcliente, fecha, monto, clave, codigoControl) {
    var value = generate(numautorizacion, numfactura, nitcliente, fecha, monto, clave);

    if (value != codigoControl)
        return "FAIL";
    else
    	return "OK";
}

//testCodigoControl('29040011007', '1503', '4189179011', '20070702', '2500', '9rCB7Sv4X29d)5k7N%3ab89p-3(5[A', '6A-DC-53-05-14');// OK - PDF CodigocontrolV7 Ejemplo base
